<?php
namespace plugins\blog\models;

use library\Slugger;
use observers\TaggingObserver;
/**
 * Post model class
 * @author Jérôme Brilland
 * @version 2.1
 * @copyright (c) 2021, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class Post extends \ActiveRecord {

    use Slugger;

    const DRAFT = 0;
    const PUBLISHED = 1;
    const HIDDEN = 2;

    protected static $tableName = 'blog_post';

    protected static $primaryKey = 'post_id';

    protected function initialize() {
        $this->attach( new TaggingObserver() );
        
        if ( !isset( $this->publication_date ) ) {
            $this->publication_date = date( 'Y-m-d' );
        }
        if ( !isset( $this->publication_time ) ) {
            $this->publication_time = date( 'H:i:s' );
        }
    }

    protected function validate() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->validateMaxLength( 'title', 100, $i18n->__( 'too_long_title', 100 ) );
        $this->validateNotEmpty( 'title', $i18n->__( 'title_cannot_be_empty' ) );
        $this->validateMaxLength( 'language', 2, $i18n->__( 'too_long_language', 2 ) );
        $this->validateMaxLength( 'slug', 100, $i18n->__( 'too_long_slug', 100 ) );
        $this->validateNotEmpty( 'body', $i18n->__( 'body_cannot_be_empty' ) );
        $this->validateInArray( 'status', [self::DRAFT, self::PUBLISHED, self::HIDDEN], $i18n->__( 'invalid_status' ) );
        $this->validateDate( 'publication_date', $i18n->__( 'invalid_publication_date' ) );
        $this->validateNotEmpty( 'publication_date', $i18n->__( 'publication_date_is_required' ) );
        $this->validateTime( 'publication_time', $i18n->__( 'invalid_publication_time' ) );
        $this->validateNotEmpty( 'publication_time', $i18n->__( 'publication_time_is_required' ) );
        if ( $this->isNewRecord() || $this->isAttributeChanged( 'slug' ) ) {
            $this->validateUnique( 'slug', $i18n->__( 'slug_already_exists' ) );
        }
    }

    protected function afterFind() {
        if ( empty( $this->summary ) ) {
            $this->summary = '';
        }
    }

    protected function beforeSave() {
        $this->title = trim( $this->title );
        if ( empty( $this->slug ) ) {
		    $this->slug = $this->slugify( $this->title );
		} else {
		    $this->slug = trim( $this->slug );
		    $this->slug = strtolower( $this->slug );
		}
        $this->user_id = \IoC::resolve( 'user' )->user_id;
    }

    public function publish() {
        $this->status = self::PUBLISHED;
        $this->publication_date = date( 'Y-m-d' );
        \IoC::resolve( 'eventmediator' )->trigger( 'blog_post_published', [$this] );
        $this->save();
    }

    public static function findLastModifiedPosts( $limit = 5 ) {
        return Post::findMany( ['post_id', 'title', 'updated_at'], ['LIMIT'=> $limit, 'ORDER' => ['updated_at' => 'DESC']] );
    }

    public static function findLastCreatedPosts( $limit = 5 ) {
        return Post::findMany( ['post_id', 'title', 'created_at'], ['LIMIT'=> $limit, 'ORDER' => ['created_at' => 'DESC']] );
    }

    public static function findLastPublishedPosts( $limit = 5 ) {
        return Post::findMany( ['post_id', 'title', 'created_at'], ['LIMIT'=> $limit, 'ORDER' => ['publication_date' => 'DESC', 'publication_time' => 'DESC']] );
    }

    public function getTags() {
        return array_column( self::getDatabase()->select( 'tag', ['[>]tagging' => 'tag_id'], ['name'], ['taggable_type' => self::getTableName(),'taggable_id' => $this->post_id] ), 'name');
    }

    public function getApprovedComments() {
        return Comment::findMany( '*', ['post_id' => $this->post_id, 'status' => Comment::APPROVED] );
    }
}
